/*
 * file:        UArt.h
 * purpose:     class for serial communication
 * project:     c++ framework for avr controllers
 * author:      Reinhard Mantey
 * copyright:   2014-2016 (c) Creative commons - by-nc-sa - some rights reserved
 *              see http://creativecommons.org/licenses/by-nc-sa/3.0/ for details
 */
#ifndef UART_H
#define UART_H
#include "halConfig.h"

#ifdef HAVE_SERIAL_COMM

namespace srd {

class ByteHandler
{
public:
  virtual ~ByteHandler();
  virtual void byteReceived(uint8_t byte) = 0;
  };


class Serial
{
protected:
  static uint8_t status;

public:
  enum BaudRate {
       BR_9600      =   UART_CALC_UBRR(9600l)
     , BR_14400     =   UART_CALC_UBRR(14400l)
     , BR_19200     =   UART_CALC_UBRR(19200l)
     , BR_28800     =   UART_CALC_UBRR(28800l)
     , BR_38400     =   UART_CALC_UBRR(38400l)
     , BR_57600     =   UART_CALC_UBRR(57600l)
     , BR_76800     =   UART_CALC_UBRR(76800l)
     , BR_115200    =   UART_CALC_UBRR(115200l)
     , BR_230400    =   UART_CALC_UBRR(230400l)
     , BR_250k      =   UART_CALC_UBRR(250000l)
     , BR_500k      =   UART_CALC_UBRR(500000l)
     , BR_1M        =   UART_CALC_UBRR(1000000l)
       };
  Serial(BaudRate br, int8_t receive, int8_t send);

protected:
  static void setSpeed(int16_t br) {
    UART_BAUDRATE_HIGH_REGISTER = br >> 8;
    UART_BAUDRATE_LOW_REGISTER  = br & 0xFF;
    };
  static void setMode() {
    UART_SET_8B1SNP;
    };
  static void enableSender(bool enable) {
    // no interrupt enabling here. It will be enabled at write cmd
    if (enable) {
       UART_ENABLE_SENDER;
       }
    else {
       UART_DISABLE_SENDER;
       }
    };
  static void enableReceiver(bool enable) {
    if (enable) {
       UART_ENABLE_RECEIVER;
       }
    else {
       UART_DISABLE_RECEIVER;
       }
    };
  static void enableSenderInterrupt(bool enable) {
    if (enable) {
       UART_ENABLE_SEND_INTERRUPT;
       }
    else {
       UART_DISABLE_SEND_INTERRUPT;
       }
    };
  static bool isFrameError()  { return status & (1 << UART_FRAME_ERROR);  };
  static bool isDataOverrun() { return status & (1 << UART_DATA_OVERRUN); };
  static bool isParityError() { return status & (1 << UART_PARITY_ERROR); };
  };


class SerialReceiver : public Serial
{
  static void receiveByteDone() __asm__(STR(UART_RECEIVE_INTERRUPT)) __attribute__((__signal__, __used__, __externally_visible__));

protected:
  static ByteHandler* byteHandler;

public:
  SerialReceiver(BaudRate br)
   : Serial(br, 1, 0) {
    }

  static void setByteHandler(ByteHandler* handler) { byteHandler = handler; };
  };


class SerialSender : public Serial
{
  static void sendByteDone() __asm__(STR(UART_SENDER_INTERRUPT)) __attribute__((__signal__, __used__, __externally_visible__));

protected:
  static const uint8_t* data2Send;
  static int16_t        sendDataSize;
  static int16_t        bytesSent;

public:
  SerialSender(BaudRate br)
   : Serial(br, 0, 1) {
    }

  static bool ready2Send() { return data2Send == nullptr; }
  static void sendBinary(const uint8_t* p, int16_t size);
  static void sendString(const uint8_t* s);
  };

};
#endif                                                     // HAVE_SERIAL_COMM
#endif
