/*
 * file:        UI.C
 * purpose:     helper functions for user interface
 * project:     c++ framework for avr controllers
 * author:      Reinhard Mantey
 * copyright:   2014-2016 (c) Creative commons - by-nc-sa - some rights reserved
 *              see http://creativecommons.org/licenses/by-nc-sa/3.0/ for details
 */
#include "UI.h"
#include <stdlib.h>

#ifdef HAVE_LCD
using namespace srd;

PROGMEM const char line1[] = "Mode: ";
PROGMEM const char line2[] = "Time: ";


static void displayTime(LCD* lcd, uint32_t* secs) {
  int8_t  h = *secs / 3600;
  int16_t r = *secs % 3600;
  int8_t  m = r / 60;
  int8_t  s = r % 60;

  lcd->writeData(' ');
  lcd->writeData(h / 10 + '0');
  lcd->writeData(h % 10 + '0');
  lcd->writeData(':');
  lcd->writeData(m / 10 + '0');
  lcd->writeData(m % 10 + '0');
  lcd->writeData(':');
  lcd->writeData(s / 10 + '0');
  lcd->writeData(s % 10 + '0');
  }


void lcdDisplay(LCD* lcd, uint32_t* secs, int8_t n, int16_t* v) {
  lcd->printLine(1, line1);
  switch (static_cast<WorkMode>(work.mode)) {
    case WorkMode::MD_Config:    lcd->print("Config");    break;
    case WorkMode::MD_Light:     lcd->print("Light");     break;
    case WorkMode::MD_CountDown: lcd->print("Countdown"); break;
    case WorkMode::MD_Done:      lcd->print("Done");      break;
    default:                     lcd->print("unknown");   break;
    }
  displayTime(lcd, secs);

  switch (static_cast<WorkMode>(work.mode)) {
    case WorkMode::MD_Config:
         lcd->endLine();
         lcd->printLine(2, line2);
         lcd->print(work.counter / 60);
         lcd->print(":");
         lcd->print(work.counter % 60);
         break;

    case WorkMode::MD_Light:
         lcd->print(" LIGHT");
         lcd->endLine();
         lcd->beginLine(2);
         for (int i=0; i < n; ++i) lcd->print(v[i]);
         break;

    case WorkMode::MD_CountDown:
         lcd->endLine();
         lcd->printLine(2, line2);
         lcd->print(work.countDown / 60);
         lcd->print(":");
         lcd->print(work.countDown % 60);
         break;

    case WorkMode::MD_Done:
         lcd->print(" D O N E");
         lcd->endLine();
         lcd->beginLine(2);
         for (int i=0; i < n; ++i) lcd->print(v[i]);
         break;
    }
  lcd->print(" ");
  lcd->endLine();
  }
#endif
