/*
 * file:        halBase.h
 * purpose:     basic definitions for hardware abstraction
 * project:     c++ framework for avr controllers
 * author:      Reinhard Mantey
 * copyright:   2003-2016 (c) Creative commons - by-nc-sa - some rights reserved
 *              see http://creativecommons.org/licenses/by-nc-sa/3.0/ for details
 */
#ifndef HALBASE_H
#define HALBASE_H

#ifdef EXTERN
#  undef EXTERN
#endif
#ifdef PLACE_GLOBALS_HERE
#  define EXTERN
#else
#  define EXTERN extern
#endif

// helper construct to get interrupt numbers from names
#define STRx(s)  #s
#define STR(s)   STRx(s)

// any class needs these includes
#include <stdint.h>
#include <avr/io.h>


/*
 * hardware abstraction is performed by different include files
 */
#if   defined(__AVR_ATmega16__)

#   include "halM16.h"

#elif defined(__AVR_ATmega48__)

#   error "a ATmega48 is to small for this lib"

#elif defined(__AVR_ATmega88__) || defined(__AVR_ATmega168__) || defined(__AVR_ATmega328__)

//  this processor family uses avr/io.h with real processor name
//  but share the hardware abstraction as ATmega88
#   ifndef __AVR_ATmega88__
#       define __AVR_ATmega88__
#   endif
#   include "halM88.h"

#elif defined(__AVR_ATmega640__) || defined(__AVR_ATmega1280__) || defined(__AVR_ATmega1281__) || defined(__AVR_ATmega2560__) || defined(__AVR_ATmega2561__)

//  this processor family uses avr/io.h with real processor name
//  but share the hardware abstraction as ATmega640
#   ifndef __AVR_ATmega640__
#       define __AVR_ATmega640__
#   endif
#   include "halM640.h"

#endif
#endif
