/*
 * file:        halM640.h
 * purpose:     hardware abstraction of atmega640
 * project:     c++ framework for avr controllers
 * author:      Reinhard Mantey
 * copyright:   2003-2016 (c) Creative commons - by-nc-sa - some rights reserved
 *              see http://creativecommons.org/licenses/by-nc-sa/3.0/ for details
 */
#ifndef HALM640_H
#define HALM640_H

#ifndef HALBASE_H
#error "don't use this file directly! Please include only halBase.h"
#endif

#if defined(__AVR_ATmega640__)

# define   TIMER0_STOP                  TCCR0B &= 0xF8
# define   TIMER0_CMPA_SET(v)           OCR0A = static_cast<uint8_t>(v)
# define   TIMER0_CMPB_SET(v)           OCR0B = static_cast<uint8_t>(v)
# define   TIMER0_MODE_NORMAL           TCCR0A &= ~(1 << WGM01 | 1 << WGM00); TCCR0B &= ~(1 << WGM02)
# define   TIMER0_MODE_PWM_PC           TCCR0A |=   0 << WGM01 | 1 << WGM00
# define   TIMER0_MODE_CTC_OCR          TCCR0A |=   1 << WGM01 | 0 << WGM00
# define   TIMER0_MODE_FAST_PWM         TCCR0A |=   1 << WGM01 | 1 << WGM00
# define   TIMER0_MODE_PWM_PC_OCR       TCCR0A |=   0 << WGM01 | 1 << WGM00; TCCR0B |= 1 << WGM02
# define   TIMER0_MODE_FAST_PWM_OCR     TCCR0A |=   1 << WGM01 | 1 << WGM00; TCCR0B |= 1 << WGM02
# define   TIMER0_ENABLE_CH_A(c,m)      TCCR0A = (TCCR0A & ~(0x03 << static_cast<uint8_t>(c))) | static_cast<uint8_t>(m) << static_cast<uint8_t>(c)
# define   TIMER0_ENABLE_CH_B(c,m)      TCCR0A = (TCCR0A & ~(0x03 << static_cast<uint8_t>(c))) | static_cast<uint8_t>(m) << static_cast<uint8_t>(c)
# define   TIMER0_PS_1                  TCCR0B |= 0 << CS02 | 0 << CS01 | 1 << CS00
# define   TIMER0_PS_8                  TCCR0B |= 0 << CS02 | 1 << CS01 | 0 << CS00
# define   TIMER0_PS_64                 TCCR0B |= 0 << CS02 | 1 << CS01 | 1 << CS00
# define   TIMER0_PS_256                TCCR0B |= 1 << CS02 | 0 << CS01 | 0 << CS00
# define   TIMER0_PS_1024               TCCR0B |= 1 << CS02 | 0 << CS01 | 1 << CS00
# define   TIMER0_PS_XF                 TCCR0B |= 1 << CS02 | 1 << CS01 | 0 << CS00
# define   TIMER0_PS_XR                 TCCR0B |= 1 << CS02 | 1 << CS01 | 1 << CS00
# define   TIMER0_COMPA_ITR             TIMER0_COMPA_vect
# define   TIMER0_COMPB_ITR             TIMER0_COMPB_vect
# define   TIMER0_OVERFLOW_ITR          TIMER0_OVF_vect

# define   TIMER1_STOP                  TCCR1B &= 0xF8
# define   TIMER1_CMPA_SET(v)           OCR1A = static_cast<uint8_t>(v)
# define   TIMER1_CMPB_SET(v)           OCR1B = static_cast<uint8_t>(v)
# define   TIMER1_CMPC_SET(v)
# define   TIMER1_ICR_SET(v)            ICR1 = v
# define   TIMER1_MODE_NORMAL           TCCR1B &= ~(1 << WGM13 | 1 << WGM12); TCCR1A &= ~(1 << WGM11 | 1 << WGM10)
# define   TIMER1_MODE_PWM_PC_8B        TCCR1B |=   0 << WGM13 | 0 << WGM12;  TCCR1A |=   0 << WGM11 | 1 << WGM10
# define   TIMER1_MODE_PWM_PC_9B        TCCR1B |=   0 << WGM13 | 0 << WGM12;  TCCR1A |=   1 << WGM11 | 0 << WGM10
# define   TIMER1_MODE_PWM_PC_10B       TCCR1B |=   0 << WGM13 | 0 << WGM12;  TCCR1A |=   1 << WGM11 | 1 << WGM10
# define   TIMER1_MODE_CTC_OCR          TCCR1B |=   0 << WGM13 | 1 << WGM12;  TCCR1A |=   0 << WGM11 | 0 << WGM10
# define   TIMER1_MODE_FAST_PWM_8B      TCCR1B |=   0 << WGM13 | 1 << WGM12;  TCCR1A |=   0 << WGM11 | 1 << WGM10
# define   TIMER1_MODE_FAST_PWM_9B      TCCR1B |=   0 << WGM13 | 1 << WGM12;  TCCR1A |=   1 << WGM11 | 0 << WGM10
# define   TIMER1_MODE_FAST_PWM_10B     TCCR1B |=   0 << WGM13 | 1 << WGM12;  TCCR1A |=   1 << WGM11 | 1 << WGM10
# define   TIMER1_MODE_PWM_PC_ICR       TCCR1B |=   1 << WGM13 | 0 << WGM12;  TCCR1A |=   1 << WGM11 | 0 << WGM10
# define   TIMER1_MODE_PWM_PC_OCR       TCCR1B |=   1 << WGM13 | 0 << WGM12;  TCCR1A |=   1 << WGM11 | 1 << WGM10
# define   TIMER1_MODE_CTC_ICR          TCCR1B |=   1 << WGM13 | 1 << WGM12;  TCCR1A |=   0 << WGM11 | 0 << WGM10
# define   TIMER1_MODE_FAST_PWM_ICR     TCCR1B |=   1 << WGM13 | 1 << WGM12;  TCCR1A |=   1 << WGM11 | 0 << WGM10
# define   TIMER1_MODE_FAST_PWM_OCR     TCCR1B |=   1 << WGM13 | 1 << WGM12;  TCCR1A |=   1 << WGM11 | 1 << WGM10
# define   TIMER1_ENABLE_CH_A(c,m)      TCCR1A = (TCCR1A & ~(0x03 << static_cast<uint8_t>(c))) | static_cast<uint8_t>(m) << static_cast<uint8_t>(c)
# define   TIMER1_ENABLE_CH_B(c,m)      TCCR1A = (TCCR1A & ~(0x03 << static_cast<uint8_t>(c))) | static_cast<uint8_t>(m) << static_cast<uint8_t>(c)
# define   TIMER1_ENABLE_CH_C(c,m)
# define   TIMER1_PS_1                  TCCR1B |= 0 << CS12 | 0 << CS11 | 1 << CS10
# define   TIMER1_PS_8                  TCCR1B |= 0 << CS12 | 1 << CS11 | 0 << CS10
# define   TIMER1_PS_64                 TCCR1B |= 0 << CS12 | 1 << CS11 | 1 << CS10
# define   TIMER1_PS_256                TCCR1B |= 1 << CS12 | 0 << CS11 | 0 << CS10
# define   TIMER1_PS_1024               TCCR1B |= 1 << CS12 | 0 << CS11 | 1 << CS10
# define   TIMER1_PS_XF                 TCCR1B |= 1 << CS12 | 1 << CS11 | 0 << CS10
# define   TIMER1_PS_XR                 TCCR1B |= 1 << CS12 | 1 << CS11 | 1 << CS10
# define   TIMER1_ENABLE_OVERFLOW       TIMSK1 |=   1 << TOIE1
# define   TIMER1_DISABLE_OVERFLOW      TIMSK1 &= ~(1 << TOIE1)
# define   TIMER1_COMPA_ITR             TIMER1_COMPA_vect
# define   TIMER1_COMPB_ITR             TIMER1_COMPB_vect
# define   TIMER1_COMPC_ITR
# define   TIMER1_OVERFLOW_ITR          TIMER1_OVF_vect

# define   TIMER2_STOP                  TCCR2B &= 0xF8
# define   TIMER2_CMPA_SET(v)           OCR2A = static_cast<uint8_t>(v)
# define   TIMER2_CMPB_SET(v)           OCR2B = static_cast<uint8_t>(v)
# define   TIMER2_MODE_NORMAL           TCCR2B &= ~(1 << WGM22); TCCR2A &= ~(1 << WGM21 | 1 << WGM20)
# define   TIMER2_MODE_PWM_PC           TCCR2B |=   0 << WGM22;  TCCR2A |=   0 << WGM21 | 1 << WGM20
# define   TIMER2_MODE_CTC_OCR          TCCR2B |=   0 << WGM22;  TCCR2A |=   1 << WGM21 | 0 << WGM20
# define   TIMER2_MODE_FAST_PWM         TCCR2B |=   0 << WGM22;  TCCR2A |=   1 << WGM21 | 1 << WGM20
# define   TIMER2_MODE_PWM_PC_OCR       TCCR2B |=   1 << WGM22;  TCCR2A |=   0 << WGM21 | 1 << WGM20
# define   TIMER2_MODE_FAST_PWM_OCR     TCCR2B |=   1 << WGM22;  TCCR2A |=   1 << WGM21 | 1 << WGM20
# define   TIMER2_ENABLE_CH_A(c,m)      TCCR2A = (TCCR2A & ~(0x03 << static_cast<uint8_t>(c))) | static_cast<uint8_t>(m) << static_cast<uint8_t>(c)
# define   TIMER2_ENABLE_CH_B(c,m)      TCCR2A = (TCCR2A & ~(0x03 << static_cast<uint8_t>(c))) | static_cast<uint8_t>(m) << static_cast<uint8_t>(c)
# define   TIMER2_PS_1                  TCCR2B |= 0 << CS22 | 0 << CS21 | 1 << CS20
# define   TIMER2_PS_8                  TCCR2B |= 0 << CS22 | 1 << CS21 | 0 << CS20
# define   TIMER2_PS_32                 TCCR2B |= 0 << CS22 | 1 << CS21 | 1 << CS20
# define   TIMER2_PS_64                 TCCR2B |= 0 << CS22 | 1 << CS21 | 1 << CS20
# define   TIMER2_PS_128                TCCR2B |= 1 << CS22 | 0 << CS21 | 0 << CS20
# define   TIMER2_PS_256                TCCR2B |= 1 << CS22 | 0 << CS21 | 0 << CS20
# define   TIMER2_PS_1024               TCCR2B |= 1 << CS22 | 0 << CS21 | 1 << CS20
# define   TIMER2_COMPA_ITR
# define   TIMER2_COMPB_ITR             TIMER2_COMP_vect
# define   TIMER2_OVERFLOW_ITR          TIMER2_OVF_vect

# define   ADC_CHANNEL_REGISTER         ADMUX
# define   ADC_TRIGGER_REGISTER         ADCSRB
# define   ADC_START_REGISTER           ADCSRA
# define   ADC_SET_AUTOTRIGGER          ADCSRA |=   1 << ADATE
# define   ADC_CLEAR_AUTOTRIGGER        ADCSRA &= ~(1 << ADATE)
# define   ADC_LEFT_ADJUSTED            ADMUX |=   1 << ADLAR
# define   ADC_RIGHT_ADJUSTED           ADMUX &= ~(1 << ADLAR)
# define   ADC_REF_VOLTAGE(ref)         ADMUX &= 0x1F; ADMUX |= ref << 6
# define   ADC_CONFIG(ps)               ADCSRA &= 0xF8; ADCSRA |= ps | 1 << ADIE
# define   ADC_INTERRUPT                ADC_vect

# define   UART_CALC_UBRR(br)           ((F_CPU+br*8)/(br*16)-1)
# define   UART_BAUDRATE_HIGH_REGISTER  UBRR0H
# define   UART_BAUDRATE_LOW_REGISTER   UBRR0L
# define   UART_STATUS_REGISTER         UCSR0A
# define   UART_DATA_REGISTER           UDR0
                                     // 1 << UPM1                   // even parity
                                     // 1 << UPM1 | 1 << UPM0       // odd parity
                                     // 1 << USBS                   // 2 stop bits
# define   UART_SET_8B1SNP              UCSR0C |= 0 << UCSZ02 | 1 << UCSZ01 | 1 << UCSZ00
# define   UART_ENABLE_SENDER           UCSR0B |=   1 << TXEN0
# define   UART_DISABLE_SENDER          UCSR0B &= ~(1 << TXEN0)
# define   UART_ENABLE_SEND_INTERRUPT   UCSR0B |=   1 << UDRIE0
# define   UART_DISABLE_SEND_INTERRUPT  UCSR0B &= ~(1 << UDRIE0)
# define   UART_ENABLE_RECEIVER         UCSR0B |=   1 << RXEN0 | 1 << RXCIE0
# define   UART_DISABLE_RECEIVER        UCSR0B &= ~(1 << RXEN0 | 1 << RXCIE0)
# define   UART_FRAME_ERROR             FE0
# define   UART_DATA_OVERRUN            DOR0
# define   UART_PARITY_ERROR            UPE0
# define   UART_RECEIVE_INTERRUPT       USART0_RX_vect
# define   UART_SENDER_INTERRUPT        USART0_UDRE_vect

#endif
#endif
